@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -27,7 +27,6 @@ import sys
 import os
 import copy
 import types
-import threading
 import cPickle
 
 ################################################################################
@@ -41,12 +40,14 @@ except ImportError:
     vc_version = 0
 
 # The tuple giving the version number.
-version_tuple = (6, 99, 12, 3, vc_version)
+version_tuple = (7, 2, 0, vc_version)
 
 # The name of this version.
-version_name = "We get the job done."
+version_name = "What's on the menu."
+# 7.3: The world (wide web) is not enough.
 
-# A string giving the version number only (7.0.1.123).
+
+# A string giving the version number only (8.0.1.123).
 version_only = ".".join(str(i) for i in version_tuple)
 
 # A verbose string giving the version.
@@ -70,6 +71,10 @@ macintosh = False
 linux = False
 android = False
 ios = False
+emscripten = False
+
+# Should we enable experimental features and debugging?
+experimental = "RENPY_EXPERIMENTAL" in os.environ
 
 import platform
 
@@ -118,11 +123,13 @@ elif platform.mac_ver()[0]:
     macintosh = True
 elif "ANDROID_PRIVATE" in os.environ:
     android = True
+elif sys.platform == 'emscripten' or "RENPY_EMSCRIPTEN" in os.environ:
+    emscripten = True
 else:
     linux = True
 
 # A flag that's true if we're on a smartphone or tablet-like platform.
-mobile = android or ios
+mobile = android or ios or emscripten
 
 # A flag that's set to true if the game directory is bundled inside a mac app.
 macapp = False
@@ -151,11 +158,13 @@ backup_blacklist = {
     "renpy.object",
     "renpy.log",
     "renpy.bootstrap",
+    "renpy.debug",
     "renpy.display",
     "renpy.display.pgrender",
     "renpy.display.scale",
     "renpy.display.presplash",
     "renpy.display.test",
+    "renpy.six",
     "renpy.text.ftfont",
     "renpy.test",
     "renpy.test.testast",
@@ -186,6 +195,8 @@ name_blacklist = {
     "renpy.display.render.IDENTITY",
     "renpy.loader.auto_lock",
     "renpy.display.screen.cprof",
+    "renpy.audio.audio.lock",
+    "renpy.audio.audio.periodic_condition",
     }
 
 
@@ -291,6 +302,7 @@ class Backup():
             mod, field = k
             setattr(mod, field, objects[v])
 
+
 # A backup of the Ren'Py modules after initial import.
 backup = None
 
@@ -308,30 +320,42 @@ def update_path(package):
     name = package.__name__.split(".")
 
     import _renpy
-    libexec = os.path.dirname(_renpy.__file__)
-    package.__path__.append(os.path.join(libexec, *name))
+    if hasattr(_renpy, '__file__'):  # .so/.dll
+        libexec = os.path.dirname(_renpy.__file__)
+        package.__path__.append(os.path.join(libexec, *name))
 
     # Also find encodings, to deal with the way py2exe lays things out.
     import encodings
     libexec = os.path.dirname(encodings.__path__[0])
     package.__path__.append(os.path.join(libexec, *name))
 
+# Replaced below.
+
+
+def plog(level, even, *args):
+    return
+
 
 def import_all():
 
     # Note: If we add a new update_path, we have to add an equivalent
     # hook in the renpython hooks dir.
 
+    # Note: If we add a new module, we need to add it to iOS.
+
     import renpy  # @UnresolvedImport
 
     update_path(renpy)
 
     import renpy.arguments  # @UnresolvedImport
 
+    import renpy.config
     import renpy.log
 
     import renpy.display
 
+    import renpy.debug
+
     # Should probably be early, as we will add it as a base to serialized things.
     import renpy.object
 
@@ -354,10 +378,15 @@ def import_all():
     import renpy.persistent
     import renpy.scriptedit
     import renpy.parser
+    import renpy.performance
+    import renpy.pydict
     import renpy.python
     import renpy.script
     import renpy.statements
 
+    global plog
+    plog = renpy.performance.log
+
     import renpy.styledata  # @UnresolvedImport
     update_path(renpy.styledata)
 
@@ -401,6 +430,9 @@ def import_all():
     import renpy.gl
     update_path(renpy.gl)
 
+    import renpy.gl2
+    update_path(renpy.gl2)
+
     import renpy.angle
     update_path(renpy.angle)
 
@@ -465,7 +497,6 @@ def import_all():
     import renpy.add_from
     import renpy.dump
 
-    import renpy.config  # depends on lots. @UnresolvedImport
     import renpy.minstore  # depends on lots. @UnresolvedImport
     import renpy.defaultstore  # depends on everything. @UnresolvedImport
 
@@ -512,6 +543,8 @@ def post_import():
     for k, v in renpy.defaultstore.__dict__.iteritems():
         renpy.store.__dict__.setdefault(k, v)
 
+    renpy.store.eval = renpy.defaultstore.eval
+
     # Import everything into renpy.exports, provided it isn't
     # already there.
     for k, v in globals().iteritems():
@@ -554,6 +587,8 @@ def reload_all():
     renpy.display.draw = None
     renpy.display.interface = None
 
+    py_compile_cache = renpy.python.py_compile_cache
+
     # Delete the store modules.
     for i in sys.modules.keys():
         if i.startswith("store") or i == "renpy.store":
@@ -567,6 +602,8 @@ def reload_all():
     # Restore the state of all modules from backup.
     backup.restore()
 
+    renpy.python.old_py_compile_cache = py_compile_cache
+
     renpy.display.im.reset_module()
 
     post_import()
