@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -23,8 +23,11 @@
 # This includes both simple settings (like the screen dimensions) and
 # methods that perform standard tasks, like the say and menu methods.
 
+from __future__ import print_function
+
 import collections
 import os
+import renpy
 
 # Can we add more config variables?
 locked = False
@@ -90,7 +93,10 @@ savedir = None
 
 # The number of screens worth of images that are allowed to
 # live in the image cache at once.
-image_cache_size = 16
+image_cache_size = None
+
+# The size of the image cache, in megabytes.
+image_cache_size_mb = 300
 
 # The number of statements we will analyze when doing predictive
 # loading. Please note that this is a total number of statements in a
@@ -359,8 +365,8 @@ rtl = False
 file_open_callback = None
 
 # The size of screenshot thumbnails. (Redefined in common/)
-thumbnail_width = None
-thumbnail_height = None
+thumbnail_width = 256
+thumbnail_height = 144
 
 # The end game transition.
 end_game_transition = None
@@ -435,6 +441,9 @@ new_character_image_argument = True
 # corresponding to a shown image.
 say_attribute_transition = None
 
+# The layer the say_attribute_transition runs on.
+say_attribute_transition_layer = None
+
 # What is the name and version of this game?
 name = ""
 version = ""
@@ -711,6 +720,12 @@ translate_files = [ ]
 # translated.
 translate_comments = [ ]
 
+# Should we trying detect user locale on first launch?
+enable_language_autodetect = False
+
+# A function from (locale, region) -> existing language.
+locale_to_language_function = None
+
 # Should we pass the full argument list to the say screen?
 old_say_args = False
 
@@ -781,9 +796,6 @@ position_viewport_side = True
 # Things that be given properties via Character.
 character_id_prefixes = [ ]
 
-# Should surfaces be cached?
-cache_surfaces = True
-
 # Should {nw} wait for voice.
 nw_voice = True
 
@@ -801,6 +813,8 @@ fast_skipping_callbacks = [ ]
 
 # Should the audio periodic callback run in its own thread.
 audio_periodic_thread = True
+if renpy.emscripten:
+    audio_periodic_thread = False
 
 # A list of fonts to preload on Ren'Py startup.
 preload_fonts = [ ]
@@ -808,6 +822,123 @@ preload_fonts = [ ]
 # Should Ren'Py process multiple ATL events in a single update?
 atl_multiple_events = True
 
+# A callback that's called when checking to see if a file is loadable.
+loadable_callback = None
+
+# How many frames should be drawn fast each time the screen needs to be
+# updated?
+fast_redraw_frames = 4
+
+# The color passed to glClearColor when clearing the screen.
+gl_clear_color = "#000"
+
+# Screens that are updated once per frame rather than once per interaction.
+per_frame_screens = [ ]
+
+# How long we store performance data for.
+performance_window = 5.0
+
+# How long does a frame have to take (to the event) to trigger profiling.
+profile_time = 1.0 / 50.0
+
+# What event do we check to see if the profile needs to be printed?
+profile_to_event = "flip"
+
+# Should we instantly zap transient displayables, or properly hide them?
+fast_unhandled_event = True
+
+# Should a fast path be used when displaying empty windows.
+fast_empty_window = True
+
+# Should all nodes participate in rollback?
+all_nodes_rollback = False
+
+# Should Ren'Py manage GC itself?
+manage_gc = True
+
+# Default thresholds that apply to garbage collection.
+gc_thresholds = (25000, 10, 10)
+
+# The threshold for a level 0 gc when we have the time.
+idle_gc_count = 2500
+
+# Should we print unreachable.
+gc_print_unreachable = "RENPY_GC_PRINT_UNREACHABLE" in os.environ
+
+# The first frame that we consider to be "idle", so we can do gc and
+# prediction.
+idle_frame = 4
+
+# Does taking the transform state go through image reference targets?
+take_state_from_target = False
+
+# Does ui.viewport set the child_size if not set?
+scrollbar_child_size = True
+
+# Should surfaces be cached?
+cache_surfaces = False
+
+# Should we optimize textures by taking the bounding rect?
+optimize_texture_bounds = True
+
+# Should we predict everything in a ConditionSwitch?
+conditionswitch_predict_all = False
+
+# Transform events to deliver each time one happens.
+repeat_transform_events = [ "show", "replace", "update" ]
+
+# How many statements should we warp through?
+warp_limit = 1000
+
+# Should dissolve statments force the use of alpha.
+dissolve_force_alpha = True
+
+# A map from a displayable prefix to a function that returns a displayable
+# corresponding to the argument.
+displayable_prefix = { }
+
+# Should we re-play a movie when it's shown again.
+replay_movie_sprites = True
+
+# A callback that is called when entering a new context.
+context_callback = None
+
+# Should we reject . and .. in filenames?
+reject_relative = True
+
+# The prefix to use on the side image.
+side_image_prefix_tag = 'side'
+
+# Do the say attributes of a hidden side image use the side image tag?
+say_attributes_use_side_image = True
+
+# Does the menu statement show a window by itself, when there is no caption?
+menu_showed_window = False
+
+# Should the menu statement produce actions instead of values?
+menu_actions = True
+
+# Should disabled menu items be included?
+menu_include_disabled = False
+
+# Should we report extraneous attributes?
+report_extraneous_attributes = True
+
+# Should we play non-loooped music when skipping?
+skip_sounds = False
+
+# Should we lint screens without parameters?
+lint_screens_without_parameters = True
+
+# If not None, a function that's used to process and modify menu arguments.
+menu_arguments_callback = None
+
+# Should Ren'PY automatically clear the screenshot?
+auto_clear_screenshot = True
+
+# Should Ren'Py allow duplicate labels.
+allow_duplicate_labels = False
+
 del os
 del collections
 
@@ -826,3 +957,5 @@ def init():
 
     global tts_function
     tts_function = renpy.display.tts.default_tts_function
+
+    import os
