@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -52,6 +52,10 @@ def default_drag_joined(drag):
     return [ (drag, 0, 0) ]
 
 
+def default_drop_allowable(drop, drags):
+    return True
+
+
 class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
     """
     :doc: drag_drop class
@@ -168,6 +172,12 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
         If false, the default, the drag is dropped onto the droppable with
         the largest degree of overlap.
 
+    `drop_allowable`
+        A callback that is called to determine whether this drop allow
+        the current drags dropped onto. It is called with two arguments.
+        The first is the Drag which determines its sensitivity.
+        The second is a list of Drags that are being dragged.
+
     Except for `d`, all of the parameters are available as fields (with
     the same name) on the Drag object. In addition, after the drag has
     been rendered, the following fields become available:
@@ -177,7 +187,9 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
 
     `w`, `h`
          The width and height of the Drag's child, in pixels.
-        """
+    """
+
+    z = 0
 
     focusable = True
 
@@ -198,6 +210,7 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
                  drag_raise=True,
                  dragged=None,
                  dropped=None,
+                 drop_allowable=default_drop_allowable,
                  drag_handle=(0.0, 0.0, 1.0, 1.0),
                  drag_joined=default_drag_joined,
                  clicked=None,
@@ -219,6 +232,7 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
         self.drag_raise = drag_raise
         self.dragged = dragged
         self.dropped = dropped
+        self.drop_allowable = drop_allowable
         self.drag_handle = drag_handle
         self.drag_joined = drag_joined
         self.clicked = clicked
@@ -290,6 +304,9 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
         # Did we move over the course of this drag?
         self.drag_moved = False
 
+        # A z index that's changed when something is raised or lowered.
+        self.z = 0
+
         if replaces is not None:
             self.x = replaces.x
             self.y = replaces.y
@@ -307,6 +324,7 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
             self.last_drop = replaces.last_drop
             self.mouse_drop = replaces.mouse_drop
             self.click_time = replaces.click_time
+            self.z = replaces.z
 
         if d is not None:
             self.add(d)
@@ -416,8 +434,8 @@ class Drag(renpy.display.core.Displayable, renpy.python.RevertableObject):
         if child is None:
             child = self.child
 
-        self.parent_width = width
-        self.parent_height = height
+        self.parent_width = renpy.display.render.render_width
+        self.parent_height = renpy.display.render.render_height
 
         cr = render(child, width, height, st, at)
         cw, ch = cr.get_size()
@@ -740,8 +758,16 @@ class DragGroup(renpy.display.layout.MultiBox):
 
     All positional parameters to the DragGroup constructor should be
     Drags, that are added to the DragGroup.
+
+
+    `min_overlap`
+        An integer which means the minimum number of pixels at the
+        overlap so that drop will be allow.
     """
 
+    z_serial = 0
+    sorted = False
+
     _list_type = renpy.python.RevertableList
 
     def __unicode__(self):
@@ -753,14 +779,21 @@ class DragGroup(renpy.display.layout.MultiBox):
 
         replaces = properties.pop("replaces", None)
 
+        min_overlap = properties.pop("min_overlap", 0)
+        self.min_overlap = min_overlap
+
         super(DragGroup, self).__init__(**properties)
 
+        self.sorted = False
+
         if replaces is not None:
             self.positions = renpy.python.RevertableDict(replaces.positions)
             self.sensitive = replaces.sensitive
+            self.z_serial = replaces.z_serial
         else:
             self.positions = renpy.python.RevertableDict()
             self.sensitive = True
+            self.z_serial = 0
 
         for i in children:
             self.add(i)
@@ -778,6 +811,8 @@ class DragGroup(renpy.display.layout.MultiBox):
         child.drag_group = self
         super(DragGroup, self).add(child)
 
+        self.sorted = False
+
     def remove(self, child):
         """
         :doc: drag_drop method
@@ -791,6 +826,14 @@ class DragGroup(renpy.display.layout.MultiBox):
         child.x = None
         super(DragGroup, self).remove(child)
 
+    def render(self, width, height, st, at):
+
+        if not self.sorted:
+            self.children.sort(key=lambda i : i.z)
+            self.sorted = True
+
+        return super(DragGroup, self).render(width, height, st, at)
+
     def event(self, ev, x, y, st):
 
         if not self.sensitive:
@@ -804,32 +847,13 @@ class DragGroup(renpy.display.layout.MultiBox):
         order given in l for those children.
         """
 
-        s = set(l)
+        self.sorted = False
 
-        offset_map = { }
+        for i in l:
+            self.z_serial += 1
+            i.z = self.z_serial
 
-        children = [ ]
-        offsets = [ ]
-
-        for i, c in enumerate(self.children):
-            if i < len(self.offsets):
-                o = self.offsets[i]
-            else:
-                o = (0, 0)
-
-            if c not in s:
-                children.append(c)
-                offsets.append(o)
-            else:
-                offset_map[c] = o
-
-        for c in l:
-            if c in offset_map:
-                children.append(c)
-                offsets.append(offset_map[c])
-
-        self.children = self._list_type(children)
-        self.offsets = self._list_type(offsets)
+        renpy.display.render.redraw(self, 0)
 
     def lower_children(self, l):
         """
@@ -837,13 +861,13 @@ class DragGroup(renpy.display.layout.MultiBox):
         the one at the bottom being the lowest.
         """
 
-        self.children.reverse()
-        self.offsets.reverse()
+        self.sorted = False
 
-        self.raise_children(l)
+        for i in l:
+            self.z_serial += 1
+            i.z = -self.z_serial
 
-        self.children.reverse()
-        self.offsets.reverse()
+        renpy.display.render.redraw(self, 0)
 
     def get_best_drop(self, joined):
         """
@@ -873,7 +897,11 @@ class DragGroup(renpy.display.layout.MultiBox):
 
                 overlap = rect_overlap_area(r1, r2)
 
-                if overlap >= max_overlap:
+                if (
+                    overlap >= max_overlap and
+                    overlap >= self.min_overlap and
+                    c.drop_allowable(c, joined)
+                ):
                     rv = c
                     max_overlap = overlap
 
@@ -898,8 +926,11 @@ class DragGroup(renpy.display.layout.MultiBox):
             if c.x is None:
                 continue
 
-            if (x >= c.x and y >= c.y and
-                    x < (c.x+c.w) and y < (c.y+c.h)):
+            if (
+                x >= c.x and y >= c.y and
+                x < (c.x + c.w) and y < (c.y + c.h) and
+                c.drop_allowable(c, joined)
+            ):
                 return c
 
     def get_children(self):
