@@ -1,4 +1,4 @@
-# Copyright 2004-2016 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -19,6 +19,8 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+from __future__ import print_function
+
 import renpy.display
 from renpy.display.render import render, Render, Matrix2D
 
@@ -93,17 +95,11 @@ class Borders(object):
     It can also provide padding information that can be supplied to the
     :propref:`padding` style property of a window or frame.
 
-    `left`
-    `top`
-    `right`
-    `bottom`
+    `left`, `top`, `right`, `bottom`
         These provide the size of the insets used by a frame, and are added
         to the padding on each side. They should zero or a positive integer.
 
-    `pad_left`
-    `pad_top`
-    `pad_right`
-    `pad_bottom`
+    `pad_left`, `pad_top`, `pad_right`, `pad_bottom`
         These are added to the padding on each side, and may be positive or
         negative. (For example, if `left` is 5 and `pad_left` is -3, the final
         padding is 2.)
@@ -145,7 +141,7 @@ class Frame(renpy.display.core.Displayable):
     :name: Frame
 
     A displayable that resizes an image to fill the available area,
-    while preserving the width and height of its borders.  is often
+    while preserving the width and height of its borders. It is often
     used as the background of a window or button.
 
     .. figure:: frame_example.png
@@ -224,6 +220,15 @@ class Frame(renpy.display.core.Displayable):
         self.right = right
         self.bottom = bottom
 
+    def __repr__(self):
+        return "<Frame {!r} ({},{},{},{}){}>".format(
+            self.image,
+            self.left,
+            self.top,
+            self.right,
+            self.bottom,
+            " tile" if self.tile else "")
+
     def __eq__(self, o):
         if not self._equals(o):
             return False
@@ -341,7 +346,8 @@ class Frame(renpy.display.core.Displayable):
 
                 if self.tile:
                     newcr = Render(cdw, cdh)
-                    newcr.clipping = True
+                    newcr.xclipping = True
+                    newcr.yclipping = True
 
                     for x in xrange(0, cdw, csw):
                         for y in xrange(0, cdh, csh):
@@ -493,8 +499,11 @@ class Frame(renpy.display.core.Displayable):
         if image is self.image:
             return self
 
+        image._unique()
+
         rv = self._copy(args)
         rv.image = image
+        rv._duplicatable = image._duplicatable
         return rv
 
     def _in_current_store(self):
@@ -508,12 +517,9 @@ class Frame(renpy.display.core.Displayable):
         return rv
 
     def visit(self):
-        return [ ]
-
-    def predict_one(self):
-        pd = renpy.display.predict.displayable
-        self.style._predict_frame(pd)
-        pd(self.image)
+        rv = [ ]
+        self.style._visit_frame(rv)
+        return rv
 
 
 class FileCurrentScreenshot(renpy.display.core.Displayable):
