@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -22,6 +22,8 @@
 # This file contains classes that handle layout of displayables on
 # the screen.
 
+from __future__ import print_function
+
 from renpy.display.render import render, Render
 import renpy.display
 
@@ -118,6 +120,9 @@ class Container(renpy.display.core.Displayable):
 
     def _duplicate(self, args):
 
+        if args and args.args:
+            args.extraneous()
+
         if not self._duplicatable:
             return self
 
@@ -779,6 +784,7 @@ class MultiBox(Container):
         spacings = [ first_spacing ] + [ spacing ] * (len(self.children) - 1)
 
         box_wrap = self.style.box_wrap
+        box_wrap_spacing = self.style.box_wrap_spacing
         xfill = self.style.xfill
         yfill = self.style.yfill
         xminimum = self.style.xminimum
@@ -880,7 +886,7 @@ class MultiBox(Container):
                 if box_wrap and remwidth - sw - padding < 0 and line:
                     maxx, maxy = layout_line(line, target_width - x, 0)
 
-                    y += line_height
+                    y += line_height + box_wrap_spacing
                     x = 0
                     line_height = 0
                     remwidth = width
@@ -922,7 +928,7 @@ class MultiBox(Container):
                 if box_wrap and remheight - sh - padding < 0:
                     maxx, maxy = layout_line(line, 0, target_height - y)
 
-                    x += line_width
+                    x += line_width + box_wrap_spacing
                     y = 0
                     line_width = 0
                     remheight = height
@@ -995,6 +1001,10 @@ class MultiBox(Container):
 
         except IgnoreLayers:
             if self.layers:
+
+                if ev.type != renpy.display.core.TIMEEVENT:
+                    renpy.display.interface.post_time_event()
+
                 return None
             else:
                 raise
@@ -1054,7 +1064,9 @@ class Window(Container):
             self.add(child)
 
     def visit(self):
-        return [ self.style.background ] + self.children
+        rv = [ ]
+        self.style._visit_window(rv.append)
+        return rv + self.children
 
     def get_child(self):
         return self.style.child or self.child
@@ -1069,10 +1081,6 @@ class Window(Container):
 
             group.members.append(self)
 
-    def predict_one(self):
-        pd = renpy.display.predict.displayable
-        self.style._predict_window(pd)
-
     def render(self, width, height, st, at):
 
         # save some typing.
@@ -1328,6 +1336,7 @@ def condition_switch_predict(switch, predict_all=None):
 
 def ConditionSwitch(*args, **kwargs):
     """
+    :name: ConditionSwitch
     :doc: disp_dynamic
     :args: (*args, predict_all=None, **properties)
 
@@ -1461,7 +1470,7 @@ def Crop(rect, child, **properties):
     :doc: disp_imagelike
     :name: Crop
 
-    This created a displayable by cropping `child` to `rect`, where
+    This creates a displayable by cropping `child` to `rect`, where
     `rect` is an (x, y, width, height) tuple. ::
 
         image eileen cropped = Crop((0, 0, 300, 300), "eileen happy")
