@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -19,6 +19,8 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+from __future__ import print_function
+
 import renpy.display
 import pygame_sdl2 as pygame
 import math
@@ -918,7 +920,10 @@ class SWDraw(object):
 
         return self.window
 
-    def should_redraw(self, needs_redraw, first_pass):
+    def can_block(self):
+        return True
+
+    def should_redraw(self, needs_redraw, first_pass, can_block):
         """
         Uses the framerate to determine if we can and should redraw.
         """
@@ -1043,6 +1048,9 @@ class SWDraw(object):
                 cx = int(cx)
                 cy = int(cy)
 
+                if cx < 0 or cy < 0:
+                    return False
+
                 cw, ch = child.get_size()
                 if cx >= cw or cy >= ch:
                     return False
@@ -1083,6 +1091,9 @@ class SWDraw(object):
 
         return rle_surf
 
+    def ready_one_texture(self):
+        return False
+
     def solid_texture(self, w, h, color):
         """
         Creates a texture filled to the edges with color.
