@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -23,6 +23,8 @@
 # It's purpose is to store in one global all of the data that would
 # be to annoying to lug around otherwise.
 
+from __future__ import print_function
+
 import renpy.display
 
 # The basepath.
@@ -124,9 +126,6 @@ class RestartContext(Exception):
     in the restarted context.
     """
 
-    def __init__(self, label):
-        self.label = label
-
 
 class RestartTopContext(Exception):
     """
@@ -134,9 +133,6 @@ class RestartTopContext(Exception):
     in the restarted context.
     """
 
-    def __init__(self, label):
-        self.label = label
-
 
 class FullRestartException(Exception):
     """
@@ -287,6 +283,7 @@ def invoke_in_new_context(callable, *args, **kwargs):  # @ReservedAssignment
     except renpy.game.JumpOutException as e:
 
         contexts[-2].force_checkpoint = True
+        contexts[-2].abnormal = True
         raise renpy.game.JumpException(e.args[0])
 
     finally:
@@ -309,7 +306,6 @@ def call_in_new_context(label, *args, **kwargs):
     new context, and saving/loading will occur in the top level
     context.
 
-    This is used to begin a second interaction with the player
     Use this to begin a second interaction with the user while
     inside an interaction.
     """
@@ -337,6 +333,7 @@ def call_in_new_context(label, *args, **kwargs):
 
     except renpy.game.JumpOutException as e:
         contexts[-2].force_checkpoint = True
+        contexts[-2].abnormal = True
         raise renpy.game.JumpException(e.args[0])
 
     finally:
