@@ -1,4 +1,4 @@
-# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -64,6 +64,7 @@ def report(msg, *args):
     print()
     print(out.encode('utf-8'))
 
+
 added = { }
 
 # Reports additional information about a message, the first time it
@@ -76,8 +77,26 @@ def add(msg):
         print(unicode(msg).encode('utf-8'))
 
 
-# Trys to evaluate an expression, announcing an error if it fails.
+# Tries to evaluate an expression, announcing an error if it fails.
 def try_eval(where, expr, additional=None):
+    """
+    :doc: lint
+
+    Tries to evaluate an expression, and writes an error to lint.txt if
+    it fails.
+
+    `where`
+        A string giving the location the expression is found. Used to
+        generate an error message of the form "Could not evaluate `expr`
+        in `where`."
+
+    `expr`
+        The expression to try evaluating.
+
+    `additional`
+        If given, an additional line of information that is addded to the
+        error message.
+    """
 
     # Make sure the expression compiles.
     try_compile(where, expr)
@@ -104,6 +123,24 @@ def try_eval(where, expr, additional=None):
 
 
 def try_compile(where, expr, additional=None):
+    """
+    :doc: lint
+
+    Tries to compile an expression, and writes an error to lint.txt if
+    it fails.
+
+    `where`
+        A string giving the location the expression is found. Used to
+        generate an error message of the form "Could not evaluate `expr`
+        in `where`."
+
+    `expr`
+        The expression to try compiling.
+
+    `additional`
+        If given, an additional line of information that is addded to the
+        error message.
+    """
 
     try:
         renpy.python.py_compile_eval_bytecode(expr)
@@ -138,17 +175,22 @@ def image_exists_imprecise(name):
         else:
             required.add(i)
 
-    for im in renpy.display.image.images:
+    for im, d in renpy.display.image.images.items():
 
         if im[0] != nametag:
             continue
 
         attrs = set(im[1:])
 
-        if [ i for i in required if i not in attrs ]:
+        if [ i for i in banned if i in attrs ]:
             continue
 
-        if [ i for i in banned if i in attrs ]:
+        li = getattr(d, "_list_attributes", None)
+
+        if li is not None:
+            attrs = attrs | set(li(im[0], required))
+
+        if [ i for i in required if i not in attrs ]:
             continue
 
         imprecise_cache.add(name)
@@ -171,18 +213,41 @@ def image_exists_precise(name):
 
     nametag = name[0]
 
-    required = set(name[1:])
+    required = set()
+    banned = set()
 
-    for im in renpy.display.image.images:
+    for i in name[1:]:
+        if i[0] == "-":
+            banned.add(i[1:])
+        else:
+            required.add(i)
+
+    for im, d in renpy.display.image.images.items():
 
         if im[0] != nametag:
             continue
 
         attrs = set(im[1:])
 
-        if attrs == required:
-            precise_cache.add(name)
-            return True
+        if attrs - required:
+            continue
+
+        rest = required - attrs
+
+        if rest:
+
+            try:
+                da = renpy.display.core.DisplayableArguments()
+                da.name=( im[0], ) + tuple(i for i in name[1:] if i in attrs)
+                da.args=tuple(i for i in name[1:] if i in rest)
+                da.lint = True
+                d._duplicate(da)
+            except:
+                continue
+
+        precise_cache.add(name)
+
+        return True
 
     return False
 
@@ -201,26 +266,17 @@ def image_exists(name, expression, tag, precise=True):
     if expression:
         return
 
-    namelist = list(name)
-    names = " ".join(namelist)
-
-    # Look for the precise name.
-    while namelist:
-        if tuple(namelist) in renpy.display.image.images:
-            return
-
-        namelist.pop()
-
-    # If we're not precise, then we have to start looking for images
-    # that we can possibly match.
-    if precise:
-        if image_exists_precise(name):
-            return
-    else:
+    if not precise:
         if image_exists_imprecise(name):
             return
 
-    report("The image named '%s' was not declared.", names)
+    # If we're not precise, then we have to start looking for images
+    # that we can possibly match.
+    if image_exists_precise(name):
+        return
+
+    report("'{}' is not an image.".format(" ".join(name)) )
+
 
 # Only check each file once.
 check_file_cache = { }
@@ -559,6 +615,13 @@ def check_label(node):
         add_arg(pi.extrakw)
 
 
+def check_screen(node):
+
+    if (node.screen.parameters is None) and renpy.config.lint_screens_without_parameters:
+        report("The screen {} has not been given a parameter list.".format(node.screen.name))
+        add("This can be fixed by writing 'screen {}():' instead.".format(node.screen.name))
+
+
 def check_styles():
     for full_name, s in renpy.style.styles.iteritems():  # @UndefinedVariable
         name = "style." + full_name[0]
@@ -739,6 +802,7 @@ def lint():
 
         elif isinstance(node, renpy.ast.Screen):
             screen_count += 1
+            check_screen(node)
 
         elif isinstance(node, renpy.ast.Define):
             check_define(node, "define")
@@ -798,6 +862,9 @@ characters per block. """.format(
 
         print()
 
+    for i in renpy.config.lint_stats_callbacks:
+        i()
+
     print()
     if renpy.config.developer and (renpy.config.original_developer != "auto"):
         print("Remember to set config.developer to False before releasing.")
