@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -90,7 +90,7 @@ class ShowIf(renpy.display.layout.Container):
         if self.show_child:
             cw, ch = cr.get_size()
             rv = renpy.display.render.Render(cw, ch)
-            rv.blit(cr, (0, 0))
+            rv.blit(cr, (0, 0), focus=self.condition)
         else:
             rv = renpy.display.render.Render(0, 0)
 
@@ -163,11 +163,13 @@ Keyword("default")
 Keyword("length")
 Keyword("allow")
 Keyword("exclude")
+Keyword("copypaste")
 Keyword("prefix")
 Keyword("suffix")
 Keyword("changed")
 Keyword("pixel_width")
 Keyword("value")
+Style("caret")
 add(text_properties)
 
 # Omit imagemap_compat for being too high level (and obsolete).
@@ -375,6 +377,7 @@ Keyword("xinitial")
 Keyword("yinitial")
 Keyword("scrollbars")
 Keyword("spacing")
+Keyword("transpose")
 Style("spacing")
 Style("xminimum")
 Style("yminimum")
@@ -436,6 +439,7 @@ def sl2add(d, replaces=None, scope=None, **kwargs):
 
     if kwargs:
         rv = Transform(child=d, **kwargs)
+        rv._main = d
 
     if (replaces is not None) and isinstance(rv, Transform):
         rv.take_state(replaces)
@@ -460,6 +464,7 @@ Keyword("droppable")
 Keyword("drag_raise")
 Keyword("dragged")
 Keyword("dropped")
+Keyword("drop_allowable")
 Keyword("drag_handle")
 Keyword("drag_joined")
 Keyword("drag_offscreen")
@@ -472,6 +477,7 @@ Keyword("alternate")
 Style("child")
 
 DisplayableParser("draggroup", renpy.display.dragdrop.DragGroup, None, many, replaces=True)
+Keyword("min_overlap")
 
 DisplayableParser("mousearea", renpy.display.behavior.MouseArea, 0, replaces=True)
 Keyword("hovered")
